#include "mem_memory.h"

Memory::Memory(sdl::Window& window, sdl::Renderer& renderer) : sdl::Game(window, renderer),
    sdl::Mixer(MIX_INIT_MID|MIX_INIT_OGG),
    font(renderer, "./fuentes/arialbd.ttf"),
    dificultad(0), musica(false), intentos(0), aciertos(0) {
    // Leer graficos
    graficos = new sdl::TextureArray(renderer, std::string("./graficos/cartas.png"), 48, 48);
    if(!musica) {
        volumenMus = 32;
        music.Volume(volumenMus);
    }
    botonsalir = new sdl::Button(renderer, sdl::Point(10,520), graficos, 53, "Salir", font, 50, sdl::Color("#ffff00ff"));
    botonsalir->SetVisible(true);
    botonabandonar = new sdl::Button(renderer, sdl::Point(440,520), graficos, 52, "Abandonar", font, 50, sdl::Color("#ffff00ff"));
    botonabandonar->SetVisible(true);
}

Memory::~Memory() {
    delete graficos;
    delete botonsalir;
    delete botonabandonar;
}

void Memory::Init() {
    std::vector<int> orden = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24};
    int n, k;
    // Desordenar, para no elegir siempre las mismas cartas en niveles fácil y medio:
    for(int i=0; i<25; i++) {
        n = sdl::random(25);
        k = orden[n];
        orden[n] = orden[i];
        orden[i] = k;
    }
    estado=0;
    quit = false;
    baraja.clear();
    carta1=carta2=-1;
    intentos=aciertos=0;
    botonsalir->SetActive(false);
    botonabandonar->SetActive(true);
    switch(dificultad) {
        case 0:
            nParejas=10; nColumnas=5;
            break;
        case 1:
            nParejas=24; nColumnas=8;
            break;
        case 2:
            nParejas=50; nColumnas=10;
            break;
    }
    nFilas=(nParejas*2)/nColumnas;
    inicio.X() = (800-50*nColumnas)/2;
    inicio.Y() = (600-50*nFilas)/2;

    // Crear baraja
    for(int i=0; i<nParejas; i++) {
        baraja.push_back(mem::Carta(graficos,orden[i]));
        baraja.push_back(mem::Carta(graficos,orden[i]));
    }
    // Mezclar
    int x;
    mem::Carta t;
    for(int i=0; i<nParejas*2; i++) {
        x=sdl::random(nParejas*2);
        t = baraja[x];
        baraja[x] = baraja[i];
        baraja[i] = t;
    }
}

void Memory::Events() {
    int fil, col;

    index=-1;
    while(event.Poll()) {
        switch(event.Type()) {
        case SDL_KEYDOWN:
            if(event.Key().keysym.sym == SDLK_ESCAPE) { quit = true; }
            break;
        case SDL_MOUSEBUTTONDOWN:
            // Verficar si se ha pulsado un botón
            if(botonsalir->Pushed(sdl::Point(event.Button().x, event.Button().y)) ||
               botonabandonar->Pushed(sdl::Point(event.Button().x, event.Button().y))) {
                quit = true;
            } else
            if(event.Button().x > inicio.X() && event.Button().y > inicio.Y() &&
               event.Button().x < inicio.X()+nColumnas*50 && event.Button().y < inicio.Y()+nFilas*50) {
                fil = (event.Button().y - inicio.Y())/50;
                col = (event.Button().x - inicio.X())/50;
                index = fil*nColumnas+col;
            }
            break;
        }
    }
}

void Memory::Update() {
    if(index==-1) return;

    switch(estado) {
    case 0:
        if(!baraja[index].Bloqueada()) {
            estado=1;
            carta1 = index;
            baraja[carta1].Mostrar();
        }
        break;
    case 1:
        if(carta1 != index && !baraja[index].Bloqueada()) {
            carta2 = index;
            baraja[carta2].Mostrar();
            if(baraja[carta1] == baraja[carta2]) {
                baraja[carta1].Bloquear();
                baraja[carta2].Bloquear();
                aciertos++;
                if(aciertos == nParejas) {
                    botonsalir->SetActive(true);
                    botonabandonar->SetActive(false);
                }
                estado = 0;
            } else {
                estado = 2;
            }
            intentos++;

        }
        break;
    case 2:
        baraja[carta1].Mostrar(false);
        baraja[carta2].Mostrar(false);
        if(!baraja[index].Bloqueada()) {
            carta1=index;
            baraja[carta1].Mostrar();
            estado = 1;
        } else {
            estado = 0;
        }
        break;
    }
}

void Memory::Render() {
    sdl::Point coord;
    std::string line;
    sdl::Color negro("#000000ff");
    sdl::Color fondo("#5a5a78ff");
    sdl::Color blanco("#ffffffff");

    // Renderizado:
    renderer.SetDrawColor(fondo);
    renderer.Clear();
    renderer.SetDrawColor(blanco);
    for(int i=0; i<nParejas*2; i++) {
        coord = sdl::Point(inicio.X()+50*(i%nColumnas), inicio.Y()+50*(i/nColumnas));
        baraja[i].Show(coord);
        //graficos->Draw(coord, baraja[i]);
    }
    // Mostrar puntuaciones:
    coord = sdl::Point(60, 2);;
    line = "Intentos " + std::to_string(intentos) + " Aciertos " + std::to_string(aciertos);
    font.SetOutline(0);
    font.SetSize(45);
    font.RenderUTF8_Blended(line, coord, negro);
    botonsalir->Render();
    botonabandonar->Render();
    renderer.Present();
}

char Memory::Run() {
    return Game::Run();
}

char Memory::Menu1(int) {
    MemMenu menu(window, renderer, "./fuentes/Hey Comic.ttf");
    menu.SetTitlePos(80,100);
    menu.SetMenuPos(80,180);
    menu.SetTitleColor({0xff,0xff,0x80,0xff});
    menu.SetTitle("-- MEMORY --");
    menu.AddOption("Seleccionar dificultad", '1');
    menu.AddOption("Sonido", '2');
    menu.AddOption("Jugar", '3');
    menu.AddOption("Creditos", '4');
    menu.AddOption("Salir", '5');
    menu.SetTextSize(35);
    return menu.Run();
}

char Memory::Menu2(int s) {
    MemMenu menu(window, renderer, "./fuentes/Hey Comic.ttf");
    menu.SetTitlePos(80,100);
    menu.SetMenuPos(80,180);
    menu.SetTitleColor({0xff,0xff,0x80,0xff});
    menu.SetTitle("-- MEMORY DIFICULTAD --");
    menu.AddOption("Fácil", '1');
    menu.AddOption("Medio", '2');
    menu.AddOption("Dificil", '3');
    menu.SetSel('1'+s);
    menu.SetTextSize(35);
    return menu.Run();
}

char Memory::Menu3(int s) {
    MemMenu menu(window, renderer, "./fuentes/Hey Comic.ttf");
    menu.SetTitlePos(80,100);
    menu.SetMenuPos(80,180);
    menu.SetTitleColor({0xff,0xff,0x80,0xff});
    menu.SetTitle("-- MEMORY MUSICA --");
    menu.AddOption("Elegir musica", '1');
    menu.AddOption("Volumen +", '2');
    menu.AddOption("Volumen -", '3');
    if(Musica()) {
        menu.AddOption("Desactivar música", '4');
    } else {
        menu.AddOption("Activar música", '4');
    }
    menu.AddOption("Volver", '5');
    menu.SetSel('1'+s);
    menu.SetTextSize(35);
    return menu.Run();
}

char Memory::Menu4(int s) {
    MemMenu menu(window, renderer, "./fuentes/Hey Comic.ttf");
    menu.SetTitlePos(80,100);
    menu.SetMenuPos(80,180);
    menu.SetTitleColor({0xff,0xff,0x80,0xff});
    menu.SetTitle("-- MEMORY SONIDO --");
    menu.AddOption("Infinte Melodic", '1');
    menu.AddOption("Melodic Techno", '2');
    menu.AddOption("Watr Fluid", '3');
    menu.SetSel('1'+s);
    menu.SetTextSize(35);
    return menu.Run();
}

void Memory::Creditos() {
    MemCreditos creditos(window, renderer, "./fuentes/arialbd.ttf");
    creditos.SetTitlePos(80,100);
    creditos.SetTextPos(80,180);
    creditos.SetTitleColor({0xff,0xff,0x80,0xff});
    creditos.SetTitle("-- MEMORY --");
    creditos.AddLine("Juego de memoria");
    creditos.AddLine("Septiembre 2024");
    creditos.AddLine("Con Clase");
    creditos.AddLine("https://conclase.net");
    creditos.AddLine("");
    creditos.AddLine("Musica desde pixabay");
    creditos.AddLine("Melodic Techno #3: Zen_Man");
    creditos.AddLine("Infinite (Melodic Rap Beat): SoundUniverseStudio");
    creditos.AddLine("WatR-Fluid: ItsWatR");
    creditos.SetTextSize(25);
    creditos.Run();
}

void MemMenu::Render() {
    sdl::Rect re;

    window.GetSize(&re.W(), &re.H());
    re.X() = 0;
    re.Y() = 0;
    renderer.SetDrawColor(0,0,0,255);
    for(int i=0; i<9; i++) {
        SDL_RenderDrawRect(renderer, re.Get());
        re.X()++; re.Y()++; re.W()-=2; re.H()-=2;
    }
    renderer.SetDrawColor(sdl::Color(0,169,157,255));
    for(int i=0; i<22; i++) {
        SDL_RenderDrawRect(renderer, re.Get());
        re.X()++; re.Y()++; re.W()-=2; re.H()-=2;
    }
    renderer.SetDrawColor(sdl::Color(41,171,226,255));
    for(int i=0; i<22; i++) {
        SDL_RenderDrawRect(renderer, re.Get());
        re.X()++; re.Y()++; re.W()-=2; re.H()-=2;
    }
    Menu::Render();
}

void MemCreditos::Render() {
    sdl::Rect re;

    window.GetSize(&re.W(), &re.H());
    re.X() = 0;
    re.Y() = 0;
    renderer.SetDrawColor(0,0,0,255);
    for(int i=0; i<9; i++) {
        SDL_RenderDrawRect(renderer, re.Get());
        re.X()++; re.Y()++; re.W()-=2; re.H()-=2;
    }
    renderer.SetDrawColor(sdl::Color(0,169,157,255));
    for(int i=0; i<22; i++) {
        SDL_RenderDrawRect(renderer, re.Get());
        re.X()++; re.Y()++; re.W()-=2; re.H()-=2;
    }
    renderer.SetDrawColor(sdl::Color(41,171,226,255));
    for(int i=0; i<22; i++) {
        SDL_RenderDrawRect(renderer, re.Get());
        re.X()++; re.Y()++; re.W()-=2; re.H()-=2;
    }
    Credits::Render();
}
