#ifndef _MEM_MEMORY
#define _MEM_MEMORY

#include <vector>
#include <cstdio>
#include <cstdlib>
#include <sdldll.h>
#include "mem_carta.h"

class Memory : public sdl::Game, public sdl::Mixer {
private:
    sdl::Font font;                 /**< Fuente para puntuaciones */
    sdl::Music music;               /**< Música */
    int dificultad;                 /**< Nivel de dificultad */
    int nParejas;                   /**< Número de parejas, en función de la dificultad */
    int nColumnas;                  /**< Número de columnas, en función de la dificultad */
    int nFilas;                     /**< Número de filas, en función de la dificultad */
    sdl::TextureArray *graficos;    /**< Gráficos */
    std::vector<mem::Carta> baraja; /**< Conjunto de parejas de cartas */
    sdl::Point inicio;              /**< Coordenadas de primera carta */
    int estado=0;                   /**< Estado del bucle del juego */
    bool musica;                    /**< Indica si la música se debe reproducir */
    int volumenMus;                 /**< Volumen de la música */
    int carta1, carta2;             /**< Índices de las cartas levantadas por el jugador */
    int intentos;                   /**< Número de intentos, parejas levantadas */
    int aciertos;                   /**< Número de aciertos, parejas levantadas coincidentes */
    sdl::Button *botonsalir;        /**< Botón de salir, activo cuando se completa el tablero */
    sdl::Button *botonabandonar;    /**< Botón de abandonar partida */
    int index;                      /**< Índice de la carta seleccionada por el usuario */

public:
    Memory(sdl::Window& window, sdl::Renderer& renderer);
    ~Memory();
    void SetDificultad(int d) { dificultad = d; }
    int GetDificultad() { return dificultad; }
    void CargaMusica(const std::string file) { music.Load(file.c_str()); music.Play(-1); if(musica) music.Resume(); else music.Pause();}
    void VolumenMas() { volumenMus+=5; if(volumenMus>128) volumenMus=128; music.Volume(volumenMus); }
    void VolumenMenos() { volumenMus-=5; if(volumenMus<0) volumenMus=0; music.Volume(volumenMus); }
    void Init();
    void Events();
    void Update();
    void Render();
    char Run();
    bool SwitchMusica() { musica = !musica; if(musica) music.Resume(); else music.Pause(); return musica; }
    bool Musica() { return musica; }
    char Menu1(int s=-1);
    char Menu2(int s=-1);
    char Menu3(int s=-1);
    char Menu4(int s=-1);
    void Creditos();
};

class MemMenu : public sdl::Menu {
public:
    MemMenu(sdl::Window& win, sdl::Renderer& ren, const std::string& font) : sdl::Menu(win, ren, font) {}
    void Render();
};

class MemCreditos : public sdl::Credits {
public:
    MemCreditos(sdl::Window& win, sdl::Renderer& ren, const std::string& font) : sdl::Credits(win, ren, font) {}
    void Render();
};
#endif // MEM_MEMORY
